==> rospell-2006.01.tar.gz: rospell-2006.01/src/rospell/utf8.h <==
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef UTF8_H
#define UTF8_H
#include <string>
#include "debug.h"

class UTF8 {
public:
	static inline unsigned utf8CharLen(char *str) {
		ASSERT(str != 0);
		unsigned char *str1 = (unsigned char *) str;
		if (!(str1[0] & 0x80))
			return 1;
		else if (((str1[0] & 0xe0) == 0xc0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) != 0x80))
			return 2;
		else if (((str1[0] & 0xf0) == 0xe0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) != 0x80))
			return 3;
		else if (((str1[0] & 0xf8) == 0xf0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) == 0x80) &&
			    ((str1[4] & 0xc0) != 0x80))
			return 4;
		return 0;
	}

	static bool isutf8RomanianChar(char *str) {
		ASSERT(str != 0);
		unsigned len = utf8CharLen(str);
		if (len != 2)
			return false;
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && (str1[1] == 0x83 || str1[1] == 0x82))
			return true;
		else if (str1[0] == 0xc5 && (str1[1] == 0x9f || str1[1] == 0x9e || str1[1] == 0xa3 || str1[1] == 0xa2))
			return true;
		else if (str1[0] == 0xc3 && (str1[1] == 0xa2 || str1[1] == 0x82 || str1[1] == 0xae || str1[1] == 0x8e))
			return true;

		return false;
	}


	static bool isutf8Punctuation(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80))
			return false;

		return !isutf8RomanianChar(str);
	}

	static inline bool isutf8alnum(char *str) {
		if (!(str[0] & 0x80))
			return std::isalnum(*str);

		if (isutf8RomanianChar(str))
			return true;

		return false;
	}

	static inline char *strtoascii(char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		char *ptr1 = str;
		char *ptr2 = retval;
		while (*ptr1 != '\0') {
			ptr1 += UTF8::chartoascii(ptr1, *ptr2);
			ptr2++;
		}
		*ptr2 = '\0';
		ASSERT(strlen(retval) <= strlen(str));
		return retval;
	}

	static inline char *strtolower(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = tolower(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static inline char *strtoupper(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = toupper(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}



	static unsigned tolower(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::tolower(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x82) {
			str1[1] = 0x83;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			str1[1] = 0x9f;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			str1[1] = 0xa3;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			str1[1] = 0xae;
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in tolower %s", str);
		str[0] ='?';
		return 1;
	}

	static unsigned toupper(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::toupper(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		LOG("%x/%x", *str1, *(str1 + 1));
		if (str1[0] == 0xc4 && str1[1] == 0x82) {
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x83) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			str1[1] = 0x9e;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			str1[1] = 0x8e;
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in tolower %s", str);
		str[0] ='?';
		return 1;
	}

	static unsigned chartoascii(char *str, char &c) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			c = str[0];
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			c = 's';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			c = 'S';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			c = 't';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			c = 'T';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			c = 'i';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			c = 'I';
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in chartoascii %s", str);
		c ='?';
		return 1;
	}

	static bool isupper(char *str) {
		unsigned char *str1 = (unsigned char *) str;
		if (*str1 != 0x80)
			return std::isupper(*str1);


		if (str1[0] == 0xc4 && str1[1] == 0x82) {
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			return true;
		}

		return false;
	}


};
#endif

==> rospell-2007.01.rc3.tar.gz: rospell-2007.01.rc3/src/rospell/utf8.h <==
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef UTF8_H
#define UTF8_H
//#include <string>
#include "debug.h"

class UTF8 {
public:
	static inline unsigned utf8CharLen(char *str) {
		ASSERT(str != 0);
		unsigned char *str1 = (unsigned char *) str;
		if (!(str1[0] & 0x80))
			return 1;
		else if (((str1[0] & 0xe0) == 0xc0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) != 0x80))
			return 2;
		else if (((str1[0] & 0xf0) == 0xe0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) != 0x80))
			return 3;
		else if (((str1[0] & 0xf8) == 0xf0) &&
			    ((str1[1] & 0xc0) == 0x80) &&
			    ((str1[2] & 0xc0) == 0x80) &&
			    ((str1[3] & 0xc0) == 0x80) &&
			    ((str1[4] & 0xc0) != 0x80))
			return 4;
		return 0;
	}

	static bool isutf8RomanianChar(char *str) {
		ASSERT(str != 0);
		unsigned len = utf8CharLen(str);
		if (len != 2)
			return false;
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && (str1[1] == 0x83 || str1[1] == 0x82))
			return true;
		else if (str1[0] == 0xc5 && (str1[1] == 0x9f || str1[1] == 0x9e || str1[1] == 0xa3 || str1[1] == 0xa2))
			return true;
		else if (str1[0] == 0xc3 && (str1[1] == 0xa2 || str1[1] == 0x82 || str1[1] == 0xae || str1[1] == 0x8e))
			return true;

		return false;
	}


	static bool isutf8Punctuation(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80))
			return false;

		return !isutf8RomanianChar(str);
	}

	static inline bool isutf8alnum(char *str) {
		if (!(str[0] & 0x80))
			return std::isalnum(*str);

		if (isutf8RomanianChar(str))
			return true;

		return false;
	}

	static inline char *strtoascii(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		char *ptr1 = (char *) str;
		char *ptr2 = retval;
		while (*ptr1 != '\0') {
			ptr1 += UTF8::chartoascii(ptr1, *ptr2);
			ptr2++;
		}
		*ptr2 = '\0';
		ASSERT(strlen(retval) <= strlen(str));
		return retval;
	}

	static inline char *strtolower(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = tolower(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}

	static inline char *strtoupper(const char *str) {
		ASSERT(str != 0);
		char *retval = new char[strlen(str) + 1];
		strcpy(retval, str);
		char *ptr = retval;
		while (*ptr != '\0') {
			unsigned len = toupper(ptr);
			ptr += len;
		}
		*ptr = '\0';
		return retval;
	}



	static unsigned tolower(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::tolower(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x82) {
			str1[1] = 0x83;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			str1[1] = 0x9f;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			str1[1] = 0xa3;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			str1[1] = 0xae;
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in tolower %s", str);
		str[0] ='?';
		return 1;
	}

	static unsigned toupper(char *str) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			str[0] = std::toupper(str[0]);
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		LOG("%x/%x", *str1, *(str1 + 1));
		if (str1[0] == 0xc4 && str1[1] == 0x82) {
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x83) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			str1[1] = 0x9e;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			str1[1] = 0x82;
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			str1[1] = 0xa2;
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			str1[1] = 0x8e;
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in tolower %s", str);
		str[0] ='?';
		return 1;
	}

	static unsigned chartoascii(char *str, char &c) {
		ASSERT(str != 0);
		if (!(str[0] & 0x80)) {
			c = str[0];
			return 1;
		}
		unsigned char *str1 = (unsigned char *) str;
		if (str1[0] == 0xc4 && str1[1] == 0x83) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc4 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9f) {
			c = 's';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			c = 'S';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xa2) {
			c = 'a';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			c = 'A';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa3) {
			c = 't';
			return 2;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			c = 'T';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0xae) {
			c = 'i';
			return 2;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			c = 'I';
			return 2;
		}

		LOG("unrecognized UTF8 sequence 0x%x, 0x%x", str[0], str[1]);
		LOG("in chartoascii %s", str);
		c ='?';
		return 1;
	}

	static bool isupper(char *str) {
		unsigned char *str1 = (unsigned char *) str;
		if (*str1 != 0x80)
			return std::isupper(*str1);


		if (str1[0] == 0xc4 && str1[1] == 0x82) {
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0x9e) {
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x82) {
			return true;
		} else if (str1[0] == 0xc5 && str1[1] == 0xa2) {
			return true;
		} else if (str1[0] == 0xc3 && str1[1] == 0x8e) {
			return true;
		}

		return false;
	}


};
#endif
